import java.util.ArrayList;
import java.util.HashMap;


public class RoundRobin extends SettingMatch{

	// 勝率および勝率順の構成文字列
	private ArrayList<Double> winPer = new ArrayList<Double>() ;
	private ArrayList<String> ranking = new ArrayList<String>() ;
	private ArrayList<Double> nowWinPerList = new ArrayList<Double>();

	
	/**
	 * コンストラクタ
	 */
	public RoundRobin( String fileName ) throws Exception{
		super(fileName,"RoundRobin");
		Util.outputString("m2011a_ver"+Simulation.VER+"\n" ,ResultFileName );
		Util.outputString("\n================ result ===============\n\n" ,ResultFileName);
	}
	
	/**
	 * バトルの組み合わせや勝敗表の表示などを行います。
	 * @throws Exception 
	 */
	public void exec() throws Exception {

		for(int i=0;i<groups.size();i++){
			Group aGroup = groups.get(i);

			// 無名タグの処理
			if( aGroup.getName().equalsIgnoreCase("") ){
				int[][] result = new int[aGroup.getPlayers().size()][aGroup.getPlayers().size()];
				for(int k=0;k<aGroup.getPlayers().size();k++){
					for(int l=0;l<aGroup.getPlayers().size();l++){
						if( k == l )
							result[k][l] = -1 ;
						else if( k > l ){
							if( result[l][k] == 1 )
								result[k][l] = 2 ;
							else if( result[l][k] == 2 )
								result[k][l] = 1 ;
							else
								result[k][l] = -1 ;
						}
						else
							result[k][l] = execMatch(aGroup,k,aGroup,l);
					}
				}
				HashMap<String,int[][]> groupResult = aGroup.getResults() ;
				groupResult.put("", result);				
				aGroup.setResults( groupResult );
			}

			// 通常の処理
			for(int j=i+1;j<groups.size();j++){
				Group bGroup = groups.get(j);
				int[][] aResult = new int[aGroup.getPlayers().size()][bGroup.getPlayers().size()];
				int[][] bResult = new int[bGroup.getPlayers().size()][aGroup.getPlayers().size()];

				for(int k=0;k<aGroup.getPlayers().size();k++){
					for(int l=0;l<bGroup.getPlayers().size();l++){
						aResult[k][l] = execMatch(aGroup,k,bGroup,l);
						if( aResult[k][l] == 1 )
							bResult[l][k] = 2 ;
						else if( aResult[k][l] == 2 )
							bResult[l][k] = 1 ;
						else
							bResult[l][k] = -1 ;
					}
				}
				HashMap<String,int[][]> groupResult = aGroup.getResults() ;
				groupResult.put(bGroup.getName(), aResult);				
				aGroup.setResults( groupResult );
				groupResult = bGroup.getResults() ;
				groupResult.put(aGroup.getName(), bResult);				
				bGroup.setResults( groupResult );
			}
		}

		outputTextResultInAllPlayer();
		outputCsvResultInAllPlayer();


		Util.saveData() ;
	}
	
	/**
	 * agのaNum番目にいる剣師と、bgのbNum番目にいる剣師を戦わせます。
	 * @param ag
	 * @param aNum
	 * @param bg
	 * @param bNum
	 * @return agのaNumが勝利なら1、bgのbNumが勝利なら2、引き分けなら-1
	 * @throws Exception 
	 */
	protected int execMatch( Group ag, int aNum, Group bg, int bNum ) throws Exception{
		int result ;
		Player a = ag.getPlayers().get(aNum);
		Player b = bg.getPlayers().get(bNum);
		Util.outputString("zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz\n" ,ResultFileName );
		Util.outputString("＠"+a.getComment()+"　VS　"+b.getComment()+"["+aNum+"@"+ag.getName()+"vs"+bNum+"@"+bg.getName()+"]\n" ,ResultFileName );
		result = battle( a,b );
		Util.outputString("＠"+a.getComment()+"　VS　"+b.getComment() ,ResultFileName);
		if( result==1)
			Util.outputString(" -> "+a.getComment()+" 勝利。\n" ,ResultFileName);
		else if( result==2)
			Util.outputString(" -> "+b.getComment()+" 勝利。\n" ,ResultFileName);
		else Util.outputString("引き分け。\n" ,ResultFileName);
		Util.outputString("\n" ,ResultFileName);
		return result ;
	}

	/**
	 * groupsの中から、名前がnameであるgroupを返します。
	 * @param name
	 * @return
	 */
	private Group getGroupByName(String name){
		for(int i=0;i<groups.size();i++)
			if( groups.get(i).getName().equalsIgnoreCase(name))
				return groups.get(i);
		
		return null ;
	}
	
	/**
	 * 全剣師の総当り表をtxt出力します。
	 */
	protected void outputTextResultInAllPlayer() {
		String outFileName = ResultTableFileName+".txt";
		Util.outputString("RESULT_m2011a_ver"+Simulation.VER+"\n" ,outFileName);

		// グループの一覧を取得
		Group[] resultGroups = new Group[groups.size()];
		for(int i=0;i<groups.size();i++)
			resultGroups[i] = getGroupByName(groups.get(i).getName());
		
		// 初めに全剣師のスペックを表示（まとめタグも
		int playerNum = 0 ;
		for(int i=0;i<resultGroups.length;i++){
			Group group = resultGroups[i];
			if( i != 0 || ! group.getName().equalsIgnoreCase("") )
				Util.outputString("["+group.getName()+"]\n" ,outFileName ) ;
			for(int j=0;j<group.getPlayers().size();j++){
				Player p = group.getPlayers().get(j) ;
				if(playerNum<100)Util.outputString(" " ,outFileName);
				if(playerNum<10)Util.outputString(" " ,outFileName);
				Util.outputString("  "+playerNum+": " ,outFileName);
				Util.outputString(p.getEntryString()+"\n" ,outFileName ) ;
				playerNum ++ ;
			}
		}

		// 列番号
		Util.outputString("zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz\n    " ,outFileName);
		for(int i=0;i<playerNum;i++){
			if(i<100) Util.outputString(" " ,outFileName);
			if(i<10) Util.outputString(" " ,outFileName);
			Util.outputString(i+"" ,outFileName);
		}
		Util.outputString("\n" ,outFileName);

		
		// 実データ
		int nowPlayerNum = 0 ;
		for(int i=0;i<resultGroups.length;i++){
			Group ag = resultGroups[i];
			// 各行を出力
			for(int j=0;j<ag.getPlayers().size();j++){
				int win=0, draw=0, lose=0 ;
				if(nowPlayerNum<100) Util.outputString(" " ,outFileName);
				if(nowPlayerNum<10) Util.outputString(" " ,outFileName);
				Util.outputString(" "+nowPlayerNum ,outFileName);
				
				// 出力対象の各グループ
				for(int k=0;k<resultGroups.length;k++){
					Group bg = resultGroups[k];
				
					int[][] result = ag.getResults().get(bg.getName());
					// 同じタグ
					if( result == null ){
						for(int l=0;l<bg.getPlayers().size();l++)
							Util.outputString("   " ,outFileName);
					}
					else {
						// 出力対象グループ内の各剣師
						for(int m=0;m<result[j].length;m++){
							if(result[j][m] == 1 ){
								Util.outputString(" ○" ,outFileName);
								win ++ ;
							}
							else if(result[j][m] == 2 ){
								Util.outputString(" ×" ,outFileName);
								lose ++ ;
							}
							else{
								Util.outputString(" －" ,outFileName);
								draw ++ ;
							}
						}
					}
				}
				if( ag.getName().equalsIgnoreCase("") )
					draw -- ;
				
				double nowWinPer = ((win+(double)draw*0.5)/(win+draw+lose))*100;
				Util.outputString( String.format(" %2d勝%2d敗%2d分 勝率：%7.3f\n" , win,lose,draw, nowWinPer ) ,outFileName);
				nowWinPerList.add(nowWinPer);
				// 勝率順ソート
				if( winPer.size() == 0 ){
					winPer.add( nowWinPer );
					ranking.add(ag.getPlayers().get(j).getEntryString());
				} else {
					for(int k=0;k<winPer.size();k++){
						if( nowWinPer > winPer.get(k) ){
							winPer.add(k, nowWinPer);
							ranking.add(k, ag.getPlayers().get(j).getEntryString());
							break ;
						}
						if( k+1 == winPer.size() ){
							winPer.add( nowWinPer );
							ranking.add(ag.getPlayers().get(j).getEntryString());
							break ;
						}
					}
				}
				nowPlayerNum ++ ;
			}
		}
		
		Util.outputString("zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz\n    " ,outFileName);
		Util.outputString("勝率順ソート\n\n" ,outFileName);
		for(int i=0;i<ranking.size();i++)
			Util.outputString(String.format("%7.3f\t"+ranking.get(i)+"\n" , winPer.get(i) ) ,outFileName);

	}

	/**
	 * 全剣師の総当りの結果をcsv出力します。
	 * @throws Exception 
	 */
	protected void outputCsvResultInAllPlayer() throws Exception {
	
		String outFileName = ResultTableFileName+".csv";
		Util.outputString("RESULT_m2011a_ver"+Simulation.VER+",,,,,,," ,outFileName);

		// グループの一覧を取得
		Group[] resultGroups = new Group[groups.size()];
		for(int i=0;i<groups.size();i++){
			resultGroups[i] = getGroupByName(groups.get(i).getName());
			// 無名タグを結果の先頭に
			if( resultGroups[i].getName().equalsIgnoreCase("") ){
				Group temp = resultGroups[i] ;
				resultGroups[i] = resultGroups[0] ;
				resultGroups[0] = temp ;
			}
		}

		// 1行目：タグカラム
		int playerNum = 0 ;
		for(int i=0;i<resultGroups.length;i++){
			Group group = resultGroups[i] ;
			Util.outputString(group.getName() ,outFileName);
			for(int j=0;j<group.getPlayers().size();j++){
				Util.outputString("," ,outFileName);
				playerNum ++ ;
			}
		}
		Util.outputString("\n" ,outFileName);
		
		// 2行目：名前カラム
		Util.outputString(",,,,,,," ,outFileName);
		for(int i=0;i<resultGroups.length;i++){
			Group group = resultGroups[i] ;
			for(int j=0;j<group.getPlayers().size();j++)
				Util.outputString(group.getPlayers().get(j).getComment()+"," ,outFileName ) ;
		}
		Util.outputString("\n" ,outFileName);
		
		// 3行目：構成カラム
		Util.outputString(",,,,,,," ,outFileName);
		for(int i=0;i<resultGroups.length;i++){
			Group group = resultGroups[i] ;
			for(int j=0;j<group.getPlayers().size();j++)
				Util.outputString(group.getPlayers().get(j).getEntryString()+"," ,outFileName ) ;
		}
		Util.outputString("\n" ,outFileName);

		Util.outputString("HP,攻,防,速,剣,名前,番号," ,outFileName);
		for(int i=0;i<playerNum;i++)
			Util.outputString(i+"," ,outFileName);
		Util.outputString("勝,分,負,勝率\n" ,outFileName);
		
		int pNum = 0 ;
		for(int i=0;i<resultGroups.length;i++){
			Group group = resultGroups[i] ;
			for(int j=0;j<group.getPlayers().size();j++){
				Player p = group.getPlayers().get(j);
				Util.outputString(p.getHp()+","+p.getAtk()+","+ p.getDef()+","+p.getSpd()+
						",\""+p.getActionStr()+"\","+p.getComment()+","+pNum+",",outFileName);
				pNum ++ ;
				
				int win=0,lose=0,draw=0;				
				// 出力対象の各グループ
				for(int k=0;k<resultGroups.length;k++){
					Group outputGroup = resultGroups[k];
				
					int[][] result = group.getResults().get(outputGroup.getName());
					// 同じタグ
					if( result == null ){
						for(int l=0;l<outputGroup.getPlayers().size();l++)
							Util.outputString("," ,outFileName);
					}
					else {
						// 出力対象グループ内の各剣師
						for(int m=0;m<result[j].length;m++){
							if(result[j][m] == 1 ){
								Util.outputString("@," ,outFileName);
								win ++ ;
							}
							else if(result[j][m] == 2 ){
								Util.outputString(".," ,outFileName);
								lose ++ ;
							}
							else{
								Util.outputString("+," ,outFileName);
								draw ++ ;
							}
						}
					}
				}
				if( group.getName().equalsIgnoreCase("") )
					draw -- ;

				Util.outputString(win+","+draw+","+lose+","+((win+(double)draw*0.5)/(win+draw+lose))*100.0+"\n" ,outFileName);
			}
		}
		Util.outputString(",,,,,,勝率," ,outFileName);
		for(int i=0;i<nowWinPerList.size();i++)
			Util.outputString(nowWinPerList.get(i)+"," ,outFileName);
	}
	


}
